;*****************************************************************
;       A collection of LCD subroutines supporting 1x16, 2x16, &
;	4x16 character LCD displays:
;
;	LCDINIT	Initializes LCD display for 4 bit data interface
;	LCDOUT	Sends ASCII data to LCD for display
;	LCDL1	Sets LCD line 1 at cursor pos. specified in W
;	LCDL2	Sets LCD line 2 at cursor pos. specified in W
;	LCDL3	Sets LCD line 1 at cursor pos. specified in W
;	LCDL4	Sets LCD line 2 at cursor pos. specified in W
;	LCDCLR	Clears LCD
;
;	See comments at the head of each subroutine below for
;	specific instructions.
;
;	Requires: LCDBUF and LCDDEL registers defined, and WAIT
;	subroutine and registers required by WAIT.  Also requires
;	that PORTB bits attached to the LCD are setup as outputs.
;
;	Assumes 4MHz clock, and LCD attached to PORTB with the
;	following attachments:
;	LCD	PIC
;	RS	RB2
;	E	RB3
;	D4	RB4
;	D5	RB5
;	D6	RB6
;	D7	RB7
;	Note: pin assignments can be modified in bit definition
;	section directly below these comments.
;
;	DH MicroSystems, Inc.
;	WWW.DHMICRO.COM
;
;*****************************************************************

;*****************************************************************
;	Bit definitions for LCD routines
;*****************************************************************
RS	EQU	0x02
E	EQU	0x03
D4	EQU	0x04
D5	EQU	0x05
D6	EQU	0x06
D7	EQU	0x07

;*****************************************************************
;	LCDINIT	(Initializes LCD display for 4 bit data interface)
;
;	Input: none
;	Output: LCD initialized for 4 bit interface
;
;	Requires: LCD registers defined above, various LCD
;	subroutines defined below, WAIT subroutine and
;	registers required by it
;
;	Executed by:
;
;	CALL	LCDINIT
;
;	DH MicroSystems, Inc.
;	WWW.DHMICRO.COM
;
;*****************************************************************
LCDINIT	BCF	PORTB,E		; E line low
	BCF	PORTB,RS	; RS line low for control

	MOVLW	0x14		; setup for 20mS delay
	CALL	WAIT		; pause 20mS for LCD to initialize

	MOVLW	b'00110000'	; 8 bit mode
	MOVWF	LCDBUF		; data moved to LCD buffer
	CALL	BITSET		; set output data lines
	CALL	STROBE		; strobe E line

	MOVLW	b'00100000'	; 4 bit mode
	MOVWF	LCDBUF		; data moved to LCD buffer
	CALL	BITSET		; set output data lines
	CALL	STROBE		; strobe E line

	MOVLW	b'00101000'	; 4 bit, 2 line LCD, 5x7 mode
	CALL	LCDOUT
	MOVLW	b'00001100'	; LCD on, cursor off, blink off
	CALL	LCDOUT
	MOVLW	b'00000001'	; clear display
	CALL	LCDOUT

	MOVLW	0x05		; setup for 5mS delay
	CALL	WAIT		; pause 5mS for LCD to clear

	BSF	PORTB,RS	; RS line high for data
	CALL	DEL_125		; pause 125uS

	RETLW	0

;*****************************************************************
;	LCDOUT	(Sends ASCII data to LCD for display)
;
;	Input: ASCII character in W register
;	Output: ASCII character displayed on LCD at current cursor
;	position
;
;	Requires: LCD registers defined above, and various LCD
;	subroutines defined below
;
;	Executed by:
;
;	MOVLW	'?'		; ? any ASCII character
;	CALL	LCDOUT
;
;	DH MicroSystems, Inc.
;	WWW.DHMICRO.COM
;
;*****************************************************************
LCDOUT	MOVWF	LCDBUF		; data moved to LCD buffer
	CALL	BITSET		; set output data lines
	CALL	STROBE		; strobe E line
	SWAPF	LCDBUF,F	; swap MS and LS nibbles
	CALL	BITSET		; set output data lines
	CALL	STROBE		; strobe E line

	RETLW	0

;*****************************************************************
;	LCDL1, LCDL2, LCDL3, & LCDL4 (Sets up to display on LCD
;	line 1, 2, 3, or 4 at column position specified in W
;	register)
;
;	Input: cursor column position in W register
;	Output: cursor moved to position specified in W register
;	on LCD line 1, 2, 3, or 4
;
;	Requires: LCD registers defined above, and various LCD
;	subroutines defined below
;
;	Executed by:
;
;	MOVLW	0x0?		; ? any number 0-Fh
;	CALL	LCDL?		; ?=1 for top line, =2 for bottom
;
;	DH MicroSystems, Inc.
;	WWW.DHMICRO.COM
;
;*****************************************************************
LCDL1	ANDLW	b'00001111'	; mask off any extraneous bits
	IORLW	b'10000000'	; setup for DD ram and line #
	MOVWF	LCDBUF		; store value in LCDBUF
	GOTO	LCDLN

LCDL2	ANDLW	b'00001111'	; mask off any extraneous bits
	IORLW	b'11000000'	; setup for DD ram and line #
	MOVWF	LCDBUF		; store value in LCDBUF
	GOTO	LCDLN

LCDL3	ANDLW	b'00001111'	; mask off any extraneous bits
	IORLW	b'10010000'	; setup for DD ram and line #
	MOVWF	LCDBUF		; store value in LCDBUF
	GOTO	LCDLN

LCDL4	ANDLW	b'00001111'	; mask off any extraneous bits
	IORLW	b'11010000'	; setup for DD ram and line #
	MOVWF	LCDBUF		; store value in LCDBUF
	GOTO	LCDLN

LCDLN	BCF	PORTB,RS	; RS line low for control
	CALL	DEL_125		; pause 125uS
	MOVF	LCDBUF,W	; recall LCDBUF to W register
	CALL	LCDOUT		; send data to LCD
	BSF	PORTB,RS	; RS line high for data
	CALL	DEL_125		; pause 125uS

	RETLW	0

;*****************************************************************
;	LCDCLR	(Clears LCD and sets cursor to line 1 pos. 1)
;
;	Input: none
;	Output: LCD cleared of characters and cursor at upper left
;
;	Requires: LCD registers defined above, and various LCD
;	subroutines defined below
;
;	Executed by:
;
;	CALL	LCDCLR
;
;	DH MicroSystems, Inc.
;	WWW.DHMICRO.COM
;
;*****************************************************************
LCDCLR	BCF	PORTB,RS	; RS line low for control
	CALL	DEL_125		; pause 125uS
	MOVLW	b'00000001'	; clear display command
	CALL	LCDOUT		; send data to LCD
	MOVLW	0x05		; setup for 5mS delay
	CALL	WAIT		; pause 5mS for LCD to clear
	BSF	PORTB,RS	; RS line high for data
	CALL	DEL_125		; pause 125uS

	RETLW	0

;*****************************************************************
;	Misc. LCD subroutines
;	
;	None of these called directly by user
;*****************************************************************
STROBE	BSF	PORTB,E		; LCD E line high
	NOP
	NOP
	NOP
	NOP
	NOP
	BCF	PORTB,E		; LCD E line low
	CALL	DEL_125		; pause 125uS

	RETLW	0

;*****************************************************************
BITSET	BCF	PORTB,D4
	BTFSC	LCDBUF,4
	BSF	PORTB,D4

	BCF	PORTB,D5
	BTFSC	LCDBUF,5
	BSF	PORTB,D5

	BCF	PORTB,D6
	BTFSC	LCDBUF,6
	BSF	PORTB,D6

	BCF	PORTB,D7
	BTFSC	LCDBUF,7
	BSF	PORTB,D7

	RETLW	0

;*****************************************************************
DEL_125	MOVLW	0x2A
	MOVWF	LCDDEL		; setup for 125uS delay
REPEAT	DECFSZ	LCDDEL,F	; decrement LCDDEL
	GOTO	REPEAT		; LCDDEL not zero? loop back

	RETLW	0